"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

PASSWORD_HMAC_SIGNATURE = "hmac:"
PASSWORD_COLUMN_NAME = "password"
USERNAME_COLUMN_NAME = "username"
USER_COOKIE_NAME = "usersecret"
USER_PLAINTEXT_COOKIE_NAME = "user"
IDENTITY_TYPE_FB_GRAPH = 'fb_graph'
IDENTITY_TYPE_GOOGLE_OAUTH2 = "google_oauth2"
IDENTITY_TYPE_OPEN_ID = "openid"